@@ -11,6 +11,7 @@
 */
 
 #include "osdaction.h"
+#include "output.h"
 
 #include <KLocalizedString>
 #include <KScreen/Config>
@@ -51,6 +52,19 @@ KScreen::SetConfigOperation *OsdAction::applyAction(const QSharedPointer<KScreen
     const OutputPtr &external = *std::find_if(outputs.cbegin(), outputs.cend(), [&internal](const auto &output) {
         return output != internal;
     });
+
+    if (config->supportedFeatures() & Config::Feature::PerOutputScaling) {
+        external->setReplicationSource(0);
+        internal->setReplicationSource(0);
+    } else {
+        if (!internal->isEnabled()) {
+            internal->setRotation(::Output::readGlobal(internal).rotation.value_or(Output::Rotation::None));
+        }
+        if (!external->isEnabled()) {
+            external->setRotation(::Output::readGlobal(external).rotation.value_or(Output::Rotation::None));
+        }
+    }
+
     switch (action) {
     case KScreen::OsdAction::Action::NoAction:
         return nullptr;
@@ -120,7 +134,6 @@ KScreen::SetConfigOperation *OsdAction::applyAction(const QSharedPointer<KScreen
         internal->setEnabled(true);
         external->setEnabled(true);
         internal->setPos(QPoint());
-        const double internalScale = internal->scale();
         ModePtr currentMode = internal->currentMode();
         if (!currentMode) { // When the internal display is not enabled
             const auto internalModesMap = internal->modes();
@@ -134,14 +147,13 @@ KScreen::SetConfigOperation *OsdAction::applyAction(const QSharedPointer<KScreen
             currentMode = bestModeIt.value();
             internal->setCurrentModeId(currentMode->id());
         }
-        external->setPos(QPoint(std::ceil(currentMode->size().width() / internalScale), 0));
+        external->setPos({copy->logicalSizeForOutputInt(*internal).width(), 0});
         break;
     }
     case KScreen::OsdAction::Action::ExtendLeft: {
         internal->setEnabled(true);
         external->setEnabled(true);
         external->setPos(QPoint());
-        const double externalScale = external->scale();
         ModePtr currentMode = external->currentMode();
         if (!currentMode) { // When the external display is not enabled
             const auto externalModesMap = external->modes();
@@ -155,7 +167,7 @@ KScreen::SetConfigOperation *OsdAction::applyAction(const QSharedPointer<KScreen
             currentMode = bestModeIt.value();
             external->setCurrentModeId(currentMode->id());
         }
-        internal->setPos(QPoint(std::ceil(currentMode->size().width() / externalScale), 0));
+        internal->setPos({copy->logicalSizeForOutputInt(*external).width(), 0});
         break;
     }
     }
